@@ -134,22 +134,26 @@ setClass("survObjOverDisp",
 # Create a generic fecundity object
 setClass("fecObj",
 		representation(fitFec = "list",
-				fecConstants = "numeric",
+				fecNames = "character",
+				fecConstants = "data.frame",
 				offspringSplitter = "data.frame",
 				meanOffspringSize = "numeric",
-				varOffspringSize = "numeric",
+				sdOffspringSize = "numeric",
 				fecByDiscrete = "data.frame",
+				offspringTypeRates = "data.frame",
 				Transform = "character")
 )
 
 # Create a generic fecundity object for multiple covariates
 setClass("fecObjMultiCov",
 		representation(fitFec = "list",
-				fecConstants = "numeric",
+				fecNames = "character",
+				fecConstants = "data.frame",
 				offspringSplitter = "data.frame",
 				meanOffspringSize = "numeric",
-				varOffspringSize = "numeric",
+				sdOffspringSize = "numeric",
 				fecByDiscrete = "data.frame",
+				offspringTypeRates = "data.frame",
 				Transform = "character")
 )
 
@@ -158,13 +162,13 @@ setClass("fecObjMultiCov",
 ## DISCRETE TRANSITION MATRIX OBJECTS ##
 # Create a generic discrete transition matrix object
 setClass("discreteTrans",
-		representation(nclasses="numeric",
-				discrete.trans="matrix",
-				discrete.surv="matrix",
-				mean.to.cont="matrix",
-				sd.to.cont="matrix",
-				distrib.to.discrete="matrix",
-				surv.to.discrete="glm"))
+		representation(nclasses = "numeric",
+				discreteTrans = "matrix",
+				discreteSurv = "matrix",
+				meanToCont = "matrix",
+				sdToCont = "matrix",
+				distribToDiscrete = "matrix",
+				survToDiscrete = "glm"))
 
 
 
@@ -817,18 +821,18 @@ createIPMTmatrix <- function(nEnvClass = 1,
 	# In case of discrete classes, take the IPM constructed above and add discrete classes defined in discreteTrans
 	if (class(discreteTrans)=="discreteTrans") {
 		
-		nDisc <- ncol(discreteTrans@discrete.surv)
-		surv.to.discrete <- predict(discreteTrans@surv.to.discrete,data.frame(size=y,size2=(y*y)),type="response")
-		cont.to.cont <- get.matrix*matrix(1-surv.to.discrete,nrow=nBigMatrix,ncol=nBigMatrix,byrow=T)
-		disc.to.disc <- discreteTrans@discrete.trans[1:nDisc,1:nDisc]*matrix(c(discreteTrans@discrete.surv),nrow=nDisc,ncol=nDisc,byrow=T)
+		nDisc <- ncol(discreteTrans@discreteSurv)
+		survToDiscrete <- predict(discreteTrans@survToDiscrete,data.frame(size=y,size2=(y*y)),type="response")
+		cont.to.cont <- get.matrix*matrix(1-survToDiscrete,nrow=nBigMatrix,ncol=nBigMatrix,byrow=T)
+		disc.to.disc <- discreteTrans@discreteTrans[1:nDisc,1:nDisc]*matrix(c(discreteTrans@discreteSurv),nrow=nDisc,ncol=nDisc,byrow=T)
 		disc.to.cont <- matrix(NA,ncol=nDisc,nrow=nBigMatrix)
 		cont.to.disc <- matrix(NA,nrow=nDisc,ncol=nBigMatrix)
 		
 		for (j in 1:nDisc) {
-			tmp<-dnorm(y,discreteTrans@mean.to.cont[j],discreteTrans@sd.to.cont[j])*h
+			tmp<-dnorm(y,discreteTrans@meanToCont[j],discreteTrans@sdToCont[j])*h
 			if (correction=="constant") tmp<-tmp/sum(tmp) 
-			disc.to.cont[,j] <- discreteTrans@discrete.surv[,j]*discreteTrans@discrete.trans["continuous",j]*tmp
-			cont.to.disc[j,] <- discreteTrans@distrib.to.discrete[j,]*surv(y,chosenCov,survObj)*surv.to.discrete
+			disc.to.cont[,j] <- discreteTrans@discreteSurv[,j]*discreteTrans@discreteTrans["continuous",j]*tmp
+			cont.to.disc[j,] <- discreteTrans@distribToDiscrete[j,]*surv(y,chosenCov,survObj)*survToDiscrete
 		}
 		
 		get.disc.matrix <- rbind(cbind(disc.to.disc,cont.to.disc),
@@ -843,7 +847,7 @@ createIPMTmatrix <- function(nEnvClass = 1,
 				ncol =1*nBigMatrix+nDisc,
 				meshpoints = y,
 				env.index = rep(1:nEnvClass,each=nBigMatrix),
-				names.discrete=rownames(discreteTrans@discrete.trans)[1:nDisc])
+				names.discrete=rownames(discreteTrans@discreteTrans)[1:nDisc])
 		
 		rc[,] <-get.disc.matrix   
 	}
@@ -903,7 +907,7 @@ createCompoundTmatrix <- function(nEnvClass = 2,
 	h<-y[2]-y[1]
 	
 	#establish how how many discrete classes there are
-	if (class(discreteTrans)=="discreteTrans") nDisc <- ncol(discreteTrans@discrete.surv) else nDisc <- 0
+	if (class(discreteTrans)=="discreteTrans") nDisc <- ncol(discreteTrans@discreteSurv) else nDisc <- 0
 	
 	#indexes for slotting in IPMs
 	indexes <- rep(1:nEnvClass,each=(nBigMatrix+nDisc))
@@ -947,25 +951,25 @@ createCompoundTmatrix <- function(nEnvClass = 2,
 		
 		# In case of discrete classes, take the IPM constructed above and add discrete classes defined in discreteTrans
 		if (class(discreteTrans)=="discreteTrans") {
-			nmes <- rownames(discreteTrans@discrete.trans)
-			surv.to.discrete <- predict(discreteTrans@surv.to.discrete,data.frame(size=y,size2=(y*y)),type="response")
-			cont.to.cont <- get.matrix*matrix(1-surv.to.discrete,nrow=nBigMatrix,ncol=nBigMatrix,byrow=T)
-			disc.to.disc <- discreteTrans@discrete.trans[1:nDisc,1:nDisc]*matrix(c(discreteTrans@discrete.surv),
+			nmes <- rownames(discreteTrans@discreteTrans)
+			survToDiscrete <- predict(discreteTrans@survToDiscrete,data.frame(size=y,size2=(y*y)),type="response")
+			cont.to.cont <- get.matrix*matrix(1-survToDiscrete,nrow=nBigMatrix,ncol=nBigMatrix,byrow=T)
+			disc.to.disc <- discreteTrans@discreteTrans[1:nDisc,1:nDisc]*matrix(c(discreteTrans@discreteSurv),
 					nrow=nDisc,ncol=nDisc,byrow=T)
 			disc.to.cont <- matrix(NA,ncol=nDisc,nrow=nBigMatrix)
 			cont.to.disc <- matrix(NA,nrow=nDisc,ncol=nBigMatrix)
 			
-			#print(discreteTrans@mean.to.cont)
-			#print(discreteTrans@sd.to.cont)
+			#print(discreteTrans@meanToCont)
+			#print(discreteTrans@sdToCont)
 			
 			for (j in 1:nDisc) {
-				tmp<-dnorm(y,discreteTrans@mean.to.cont[j],discreteTrans@sd.to.cont[j])*h
+				tmp<-dnorm(y,discreteTrans@meanToCont[j],discreteTrans@sdToCont[j])*h
 				if (correction=="constant") tmp<-tmp/sum(tmp)
 				
 				#print(tmp)
 				
-				disc.to.cont[,j] <- discreteTrans@discrete.surv[,j]*discreteTrans@discrete.trans["continuous",j]*tmp
-				cont.to.disc[j,] <- discreteTrans@distrib.to.discrete[j,]*surv(y,cov=k,survObj)*surv.to.discrete
+				disc.to.cont[,j] <- discreteTrans@discreteSurv[,j]*discreteTrans@discreteTrans["continuous",j]*tmp
+				cont.to.disc[j,] <- discreteTrans@distribToDiscrete[j,]*surv(y,cov=k,survObj)*survToDiscrete
 			}
 			
 			get.matrix <- rbind(cbind(disc.to.disc,cont.to.disc),cbind(disc.to.cont,cont.to.cont))
@@ -1013,7 +1017,6 @@ createCompoundTmatrix <- function(nEnvClass = 2,
 #Returns -
 #  an IPM object
 
-
 createIPMFmatrix <- function(fecObj,
 		nEnvClass = 1,
 		nBigMatrix = 50,
@@ -1037,46 +1040,48 @@ createIPMFmatrix <- function(fecObj,
 	
 	fecObj@fecConstants[is.na(fecObj@fecConstants)] <- 1
 	
-	fecValues <- matrix(c(rep(1,length(fecObj@fitFec)),fecObj@fecConstants),
+	#fecundity rates
+	fecValues <- matrix(c(rep(1,length(fecObj@fitFec)),unlist(fecObj@fecConstants)),
 			ncol=nBigMatrix,nrow=length(fecObj@fitFec)+
 					length(fecObj@fecConstants))
-	
-	#print(fecValues)
-	
+	#rownames(fecValues) <- c(fecObj@fecNames,names(fecObj@fecConstants))
 	for (i in 1:length(fecObj@fitFec)) fecValues[i,] <- predict(fecObj@fitFec[[i]],newd,type="response")
-	
-	#Transforms
 	if (length(grep("log",fecObj@Transform))>0) for (i in grep("log",fecObj@Transform)) fecValues[i,]<-exp(fecValues[i,])
 	if (length(grep("sqrt",fecObj@Transform))>0) for (i in grep("sqrt",fecObj@Transform)) fecValues[i,]<-(fecValues[i,])^2
 	if (length(grep("-1",fecObj@Transform))>0) for (i in grep("-1",fecObj@Transform)) fecValues[i,]<-fecValues[i,]+1
-	fecValues[!is.finite(fecValues)] <- exp(200)
-	prodFecValues<-apply(fecValues,2,prod)
+	#fecValues[!is.finite(fecValues)] <- exp(200)
+	prodFecValues <- apply(fecValues[which(fecObj@offspringTypeRates[,"continuous"]==1),],2,prod)*unlist(fecObj@offspringSplitter["continuous"])
+	
+	#print(prodFecValues)	
 	
 	#Kids normal dist
-	tmp<-dnorm(y,fecObj@meanOffspringSize,sqrt(fecObj@varOffspringSize))*h
+	tmp<-dnorm(y,fecObj@meanOffspringSize,(fecObj@sdOffspringSize))*h
 	if (integrateType=="cumul") { 
-		tmp1 <- dnorm(b,fecObj@meanOffspringSize,sqrt(fecObj@varOffspringSize))
+		tmp1 <- dnorm(b,fecObj@meanOffspringSize,(fecObj@sdOffspringSize))
 		tmp <- tmp1[2:(nBigMatrix+1)]-tmp1[1:nBigMatrix]
 	}
 	if (correction=="constant") tmp<-tmp/sum(tmp)
-	to.cont<-tmp%*%t(as.numeric(fecObj@offspringSplitter["continuous"])*prodFecValues)
+	to.cont<-tmp%*%t(prodFecValues)
 	get.matrix <- to.cont
-	nDisc <- length(fecObj@offspringSplitter)-1
 	
+	#discrete classes
+	nDisc <- length(fecObj@offspringSplitter)-1
 	namesDiscrete <- "NA"
 	if (nDisc>0) {
 		namesDiscrete <- colnames(fecObj@offspringSplitter[1:nDisc])
-		
-		to.discrete <- as.numeric(fecObj@offspringSplitter)[1:nDisc]%*%t(prodFecValues)
+		to.discrete <- matrix(NA,nrow=nDisc,ncol=nBigMatrix)
+		for (i in 1:nDisc) to.discrete[i,] <- apply(fecValues[which(fecObj@offspringTypeRates[,namesDiscrete[i]]==1),],2,prod)*unlist(fecObj@offspringSplitter[namesDiscrete[i]])
 		
 		from.discrete <- matrix(0,ncol=nDisc,nrow=nDisc+nBigMatrix)
 		if (names(fecObj@fecByDiscrete)[1]!="NA.") {
 			if (sum(names(fecObj@fecByDiscrete)!=namesDiscrete)>0) stop ("Error - the names of the discrete classes as you provided for the data.frame fecByDiscrete are not 100% the same discrete class names in your data.frame offspringSplitter. They should also be in alphabetical order.")
-			from.discrete <- c(as.numeric(fecObj@offspringSplitter)[1:nDisc],as.numeric(fecObj@offspringSplitter)[nDisc+1]*tmp)%*%as.matrix(fecObj@fecByDiscrete)
+			from.discrete <- c(as.numeric(fecObj@offspringSplitter)[1:nDisc],as.numeric(fecObj@offspringSplitter["continuous"])*tmp)%*%as.matrix(fecObj@fecByDiscrete)
 		}
 		get.matrix <- cbind(from.discrete,rbind(to.discrete,to.cont))
 	}
 	
+	#print(colSums(get.matrix))
+	
 	#warning about negative numbers
 	if (min(get.matrix)<0) { 
 		print("Warning: fertility values < 0 exist in matrix, consider transforms. Negative values set to zero") 
@@ -1196,6 +1201,92 @@ createCompoundFmatrix <- function(nEnvClass = 2,
 
 
 
+#Function creates a single F.IPM (fecundity transitions only)
+#for a chosen size range, env category (single one at a time)
+#and survival and fecundity objects (assume survival 
+#precedes growth; could do otherwise...)
+#
+#Parameters -
+#   nEnvClass - the number of env classes, cannot be !=1, defaults to 1 
+#   nBigMatrix - the number of size bins in the model
+#   minSize - lower end of desired size range
+#   maxSize - upper end of desired size range
+#   chosenCov - current level of covariate
+#   fecObj - a fecundity object
+#   integrateType - options include "midpoint" "cumul" 
+#   etc...
+#Returns -
+#  an IPM object
+
+
+createIPMCmatrix <- function(clonalObj,
+		nEnvClass = 1,
+		nBigMatrix = 50,
+		minSize = -1,
+		maxSize = 50,
+		chosenCov = 1,
+		integrateType="midpoint",
+		correction="none") {
+	
+	rc <- createIPMFmatrix(fecObj=clonalObj,
+						  nEnvClass=nEnvClass,
+						  nBigMatrix=nBigMatrix,
+						  minSize=minSize,
+						  maxSize=maxSize,
+						  chosenCov=chosenCov,
+						  integrateType=integrateType,
+						  correction=correction)
+	
+	return(rc)
+}
+
+
+
+
+
+#Function creates a combo of F.IPMs for a chosen
+#size range, with range env categories and transitions 
+#between them
+#and growth and survival objects
+#
+#Parameters -
+#   nEnvClass - the number of env classes, defaults to 2, should match dim envMatrix
+#   nBigMatrix - the number of size bins in the model
+#   minSize - lower end of desired size range
+#   maxSize - upper end of desired size range
+#   envMatrix - a matrix describing transiions between env
+#   fecObj - a fecundity object
+#   integrateType - NOT YET IMPLEMENTED
+#
+#Returns -
+#  an IPM object
+
+
+createCompoundCmatrix <- function(nEnvClass = 2,
+		nBigMatrix = 50,
+		minSize = -1,
+		maxSize = 50,
+		envMatrix,
+		clonalObj,
+		integrateType="midpoint",
+		correction="none") {
+	
+		rc <- createCompoundFmatrix(nEnvClass = nEnvClass,
+				nBigMatrix = nBigMatrix,
+				minSize = minSize,
+				maxSize = maxSize,
+				envMatrix = envMatrix,
+				fecObj = clonalObj,
+				integrateType=integrateType,
+				correction=correction)
+	
+	return(rc) 
+}
+
+
+
+
+
 # For a single Tmatrix (!not compound and no discrete stages!), this functions makes a series
 # of diagnostic plots - this is defined for growthObj,
 # growthObjIncr - modification required
@@ -1211,7 +1302,7 @@ createCompoundFmatrix <- function(nEnvClass = 2,
 # Returns - 
 #
 diagnosticsTmatrix <- function(Tmatrix,growObj,survObj, dff, integrateType="midpoint", correction="none") {
-
+	
 	
 	#Print the range of the Tmatrix (should be on 0-1)
 	print("Range of Tmatrix is "); print(range(c(Tmatrix)))
@@ -1254,85 +1345,85 @@ diagnosticsTmatrix <- function(Tmatrix,growObj,survObj, dff, integrateType="midp
 	loctest <- floor(quantile(1:Tmatrix@nBigMatrix,c(0.25,0.5,0.75)))
 	h <- diff(Tmatrix@meshpoints)[1]
 	testSizes <- seq(min(Tmatrix@meshpoints),max(Tmatrix@meshpoints),length=5000)
+	
+	for (j in 1:3) {
+		#prob survive
+		ps <- surv(Tmatrix@meshpoints[loctest[j]],Tmatrix@env.index[1],survObj)
 		
-		for (j in 1:3) {
-			#prob survive
-			ps <- surv(Tmatrix@meshpoints[loctest[j]],Tmatrix@env.index[1],survObj)
-			
-			#set up for prediction
-			newd <- data.frame(size=Tmatrix@meshpoints[loctest[j]],
+		#set up for prediction
+		newd <- data.frame(size=Tmatrix@meshpoints[loctest[j]],
 				size2=Tmatrix@meshpoints[loctest[j]]^2,
 				covariate=Tmatrix@env.index[1])
-			if(length(grep("logsize",names(growObj@fit$coefficients))))
-				newd$logsize=log(Tmatrix@meshpoints[loctest[j]])
-			
-			if (length(growObj@fit$model$covariate)>0)
-				if (is.factor(growObj@fit$model$covariate))
-					newd$covariate <- as.factor(newd$covariate)
-			
-			#predict mean
-			if (length(grep("decline",tolower(as.character(class(growObj)))))>0 | 
-					length(grep("trunc",tolower(as.character(class(growObj)))))>0) { 
-				mux <- .predictMuX(growObj,newd)
-			} else  {
-				mux <- predict(growObj@fit,newd,type="response"); #print("yes")	
-			}
-			
-			#add to size if it is a incr object, taking exp if appropirate
-			if (length(grep("incr",tolower(as.character(class(growObj)))))>0 & 
-					length(grep("logincr",tolower(as.character(class(growObj)))))==0) {
-				mux <- Tmatrix@meshpoints[loctest[j]]+mux; #print("no")
-			}
-			
-			#define variance 
-			if (length(grep("decline",tolower(as.character(class(growObj)))))==0 & 
-					length(grep("trunc",tolower(as.character(class(growObj)))))==0) { 
-				sigmax2 <- summary(growObj@fit)$sigma^2 
-			} else { 
-				sigmax2 <- growObj@fit$sigmax2
-				var.exp.coef<-growObj@fit$var.exp.coef
-				sigmax2<-sigmax2*exp(2*(var.exp.coef*mux))
-				#overwrite with simpler for trunc
-				if (length(grep("trunc",tolower(as.character(class(growObj)))))>0) sigmax2 <- growObj@fit$sigmax2
-			} 
-			
-			
-			#plot template
-			plot(Tmatrix@meshpoints,Tmatrix@.Data[,loctest[j]]/h/ps, type="n",
-					xlim=range(Tmatrix@meshpoints[loctest[j]]+
-									c(-3.5*sqrt(sigmax2),+3.5*sqrt(sigmax2))),
-					xlab="Size next", ylab="pdf")
-			if (j==1) title("Numerical resolution and growth")
-			for (k in 1:length(Tmatrix@meshpoints)) {
-				points(c(Tmatrix@meshpoints[k])+c(-h/2,h/2),
-						rep(Tmatrix@.Data[k,loctest[j]],2)/h/ps,type="l")
-				points(rep(Tmatrix@meshpoints[k]+h/2,2),c(0,Tmatrix@.Data[k,loctest[j]]/h/ps),type="l",lty=1)
-				points(rep(Tmatrix@meshpoints[k]-h/2,2),c(0,Tmatrix@.Data[k,loctest[j]]/h/ps),type="l",lty=1)
-			}
-			#plot using mean 
-			if (length(grep("logincr",tolower(as.character(class(growObj)))))==0 & 
-					length(grep("trunc",tolower(as.character(class(growObj)))))==0) {
-					points(testSizes,dnorm(testSizes,mux,sqrt(sigmax2)),type="l",col=2) 
-				} else { 
-				 	if (length(grep("trunc",tolower(as.character(class(growObj)))))>0) { 
-						#print("here"); print(mux); print(Tmatrix@meshpoints[loctest[j]])
-						require(truncnorm)
-						points(testSizes,dtruncnorm(testSizes,
-										a=Tmatrix@meshpoints[loctest[j]],b=Inf,
-										mean=mux,sd=sqrt(sigmax2)),type="l",col=2)
-			
-				} else {
+		if(length(grep("logsize",names(growObj@fit$coefficients))))
+			newd$logsize=log(Tmatrix@meshpoints[loctest[j]])
+		
+		if (length(growObj@fit$model$covariate)>0)
+			if (is.factor(growObj@fit$model$covariate))
+				newd$covariate <- as.factor(newd$covariate)
+		
+		#predict mean
+		if (length(grep("decline",tolower(as.character(class(growObj)))))>0 | 
+				length(grep("trunc",tolower(as.character(class(growObj)))))>0) { 
+			mux <- .predictMuX(growObj,newd)
+		} else  {
+			mux <- predict(growObj@fit,newd,type="response"); #print("yes")	
+		}
+		
+		#add to size if it is a incr object, taking exp if appropirate
+		if (length(grep("incr",tolower(as.character(class(growObj)))))>0 & 
+				length(grep("logincr",tolower(as.character(class(growObj)))))==0) {
+			mux <- Tmatrix@meshpoints[loctest[j]]+mux; #print("no")
+		}
+		
+		#define variance 
+		if (length(grep("decline",tolower(as.character(class(growObj)))))==0 & 
+				length(grep("trunc",tolower(as.character(class(growObj)))))==0) { 
+			sigmax2 <- summary(growObj@fit)$sigma^2 
+		} else { 
+			sigmax2 <- growObj@fit$sigmax2
+			var.exp.coef<-growObj@fit$var.exp.coef
+			sigmax2<-sigmax2*exp(2*(var.exp.coef*mux))
+			#overwrite with simpler for trunc
+			if (length(grep("trunc",tolower(as.character(class(growObj)))))>0) sigmax2 <- growObj@fit$sigmax2
+		} 
+		
+		
+		#plot template
+		plot(Tmatrix@meshpoints,Tmatrix@.Data[,loctest[j]]/h/ps, type="n",
+				xlim=range(Tmatrix@meshpoints[loctest[j]]+
+								c(-3.5*sqrt(sigmax2),+3.5*sqrt(sigmax2))),
+				xlab="Size next", ylab="pdf")
+		if (j==1) title("Numerical resolution and growth")
+		for (k in 1:length(Tmatrix@meshpoints)) {
+			points(c(Tmatrix@meshpoints[k])+c(-h/2,h/2),
+					rep(Tmatrix@.Data[k,loctest[j]],2)/h/ps,type="l")
+			points(rep(Tmatrix@meshpoints[k]+h/2,2),c(0,Tmatrix@.Data[k,loctest[j]]/h/ps),type="l",lty=1)
+			points(rep(Tmatrix@meshpoints[k]-h/2,2),c(0,Tmatrix@.Data[k,loctest[j]]/h/ps),type="l",lty=1)
+		}
+		#plot using mean 
+		if (length(grep("logincr",tolower(as.character(class(growObj)))))==0 & 
+				length(grep("trunc",tolower(as.character(class(growObj)))))==0) {
+			points(testSizes,dnorm(testSizes,mux,sqrt(sigmax2)),type="l",col=2) 
+		} else { 
+			if (length(grep("trunc",tolower(as.character(class(growObj)))))>0) { 
+				#print("here"); print(mux); print(Tmatrix@meshpoints[loctest[j]])
+				require(truncnorm)
+				points(testSizes,dtruncnorm(testSizes,
+								a=Tmatrix@meshpoints[loctest[j]],b=Inf,
+								mean=mux,sd=sqrt(sigmax2)),type="l",col=2)
+				
+			} else {
 				points(testSizes,dlnorm(testSizes-Tmatrix@meshpoints[loctest[j]],mux,
 								sqrt(sigmax2)),type="l",col=2)
 			}}
 		
-			if (j==1) legend("topright", legend=c("Small"),col="white",lty=1,bty="n")
-			if (j==2) legend("topright", legend=c("Medium"),col="white",lty=1,bty="n")
-			if (j==3) legend("topright", legend=c("Large"),col="white",lty=1,bty="n")
-			
-		}
-	
+		if (j==1) legend("topright", legend=c("Small"),col="white",lty=1,bty="n")
+		if (j==2) legend("topright", legend=c("Medium"),col="white",lty=1,bty="n")
+		if (j==3) legend("topright", legend=c("Large"),col="white",lty=1,bty="n")
 		
+	}
+	
+	
 }
 
 
@@ -1341,6 +1432,7 @@ diagnosticsTmatrix <- function(Tmatrix,growObj,survObj, dff, integrateType="midp
 
 
 
+
 ### FUNCTIONS FOR EXTRACTING INTEGRATED DEMOGRAPHIC MEASURES ########################
 ### i.e. life-expectancy and passage time
 
@@ -1376,8 +1468,8 @@ varLifeExpect <- function(IPMmatrix){
 
 
 
-
-
+		
+		
 #Generic for survivorship
 #parameters - IPMmatrix - an IPM
 #           - size1 - a size at age 1
@@ -1385,10 +1477,9 @@ varLifeExpect <- function(IPMmatrix){
 # returns - a list including the survivorship up to the max age,
 #                      this broken down by stage,
 #                       and mortality over age 
-
-## WON'T WORK WITH DISCRETE STAGES AS IS!!
-
-survivorship <- function(IPMmatrix, size1, maxAge=300){
+		
+		
+survivorship <- function(IPMmatrix, loc, maxAge=300){
 			nBigMatrix <- length(IPMmatrix@.Data[1,])
 			#n <- IPMmatrix@nEnvClass*nBigMatrix
 			n <- nBigMatrix
@@ -1396,10 +1487,10 @@ survivorship <- function(IPMmatrix, size1, maxAge=300){
 			stage.agesurv <- matrix(NA,n,maxAge)
 			surv.curv <- rep (NA,maxAge)
 			
-			#identify the starting size you want to track
-			loc <- which(abs(size1-IPMmatrix@meshpoints)==min(abs(size1-IPMmatrix@meshpoints)),arr.ind=T)
+			#identify the starting size you want to track - removed - specify bin directly
+			#loc <- which(abs(size1-IPMmatrix@meshpoints)==min(abs(size1-IPMmatrix@meshpoints)),arr.ind=T)[1]
 			popvec <- matrix(0,n,1)
-			popvec[loc,1] <- 1
+			popvec[floor(loc),1] <- 1
 			
 			for (a in 1:maxAge) {
 				surv.curv[a]<-sum(A1[,loc]); 
@@ -1411,9 +1502,9 @@ survivorship <- function(IPMmatrix, size1, maxAge=300){
 			
 			return(list(surv.curv=surv.curv,stage.agesurv=stage.agesurv, mortality = mortality))
 		}
-
-
-
+		
+		
+		
 
 
 #Generic for first passage time 
@@ -1834,6 +1925,7 @@ largeMatrixCalc <- function(Tmatrix, Fmatrix, tol = 1.e-8){
 
 sensParams <- function(growObj,survObj,fecObj,
 		nBigMatrix,minSize,maxSize,
+		discreteTrans =1,
 		integrateType="midpoint",
 		correction="none") { 
 	
@@ -1844,24 +1936,23 @@ sensParams <- function(growObj,survObj,fecObj,
 	fec.coeff.names <- c()
 	for (i in 1:length(fecObj@fitFec)){
 		nfec <- nfec + length(fecObj@fitFec[[i]]$coefficients)	
-	    fec.coeff.names <- c(fec.coeff.names,
+		fec.coeff.names <- c(fec.coeff.names,
 				paste("reprod",i,names(fecObj@fitFec[[i]]$coefficients)))
-	
+		
 	}
 	
-	#print(fec.coeff.names)	
 	
 	# define numbers of parameters
 	npar <- length(growObj@fit$coeff)+1+  #one extra for variance
 			length(survObj@fit$coeff)+
 			(sum(!is.na(fecObj@fecConstants)))+nfec	
 	#print(npar)
-		
+	
 	# create a named vector to hold them 
 	elam <- rep(0,npar);
 	if ((sum(!is.na(fecObj@fecConstants)))>0) { 
 		nmes <- c(paste("grow",names(growObj@fit$coeff)), "sd growth",paste("surv",names(survObj@fit$coeff)),
-				paste("reprod constant",which(!is.na(fecObj@fecConstants), arr.ind=TRUE)),
+				paste("reprod constant",which(!is.na(fecObj@fecConstants), arr.ind=TRUE)[,2]),
 				fec.coeff.names) 
 	} else { 
 		nmes <- c(paste("grow",names(growObj@fit$coeff)), "sd growth",paste("surv",names(survObj@fit$coeff)),				
@@ -1874,7 +1965,8 @@ sensParams <- function(growObj,survObj,fecObj,
 	
 	# build the IPM and get the lamdba value
 	Tmatrix <- createIPMTmatrix(nBigMatrix = nBigMatrix, minSize=minSize,maxSize=maxSize,
-			growObj=growObj,survObj=survObj, integrateType=integrateType, correction=correction)
+			growObj=growObj,survObj=survObj, discreteTrans=discreteTrans,
+			integrateType=integrateType, correction=correction)
 	Fmatrix <- createIPMFmatrix(nBigMatrix = nBigMatrix, minSize=minSize,maxSize=maxSize,
 			fecObj=fecObj,integrateType=integrateType, correction=correction)
 	IPM <- Tmatrix+Fmatrix
@@ -1886,7 +1978,8 @@ sensParams <- function(growObj,survObj,fecObj,
 	for (param.test in 1:length(growObj@fit$coeff)){
 		growObj@fit$coefficients[param.test] <- growObj@fit$coefficients[param.test]*(1+delta);
 		Tmatrix <- createIPMTmatrix(nBigMatrix = nBigMatrix, minSize=minSize,maxSize=maxSize,
-				growObj=growObj,survObj=survObj, integrateType=integrateType, correction=correction)
+				growObj=growObj,survObj=survObj, discreteTrans=discreteTrans,
+				integrateType=integrateType, correction=correction)
 		Fmatrix <- createIPMFmatrix(nBigMatrix = nBigMatrix, minSize=minSize,maxSize=maxSize,
 				fecObj=fecObj, integrateType=integrateType, correction=correction)
 		IPM <- Tmatrix+Fmatrix
@@ -1900,7 +1993,8 @@ sensParams <- function(growObj,survObj,fecObj,
 	resids <- growObj@fit$residuals
 	growObj@fit$residuals <- rnorm(length(growObj@fit$residuals),0,sd(growObj@fit$residuals)*(1+delta))
 	Tmatrix <- createIPMTmatrix(nBigMatrix = nBigMatrix, minSize=minSize,maxSize=maxSize,
-			growObj=growObj,survObj=survObj, integrateType=integrateType, correction=correction)
+			growObj=growObj,survObj=survObj, discreteTrans=discreteTrans,
+			integrateType=integrateType, correction=correction)
 	Fmatrix <- createIPMFmatrix(nBigMatrix = nBigMatrix, minSize=minSize,maxSize=maxSize,
 			fecObj=fecObj, integrateType=integrateType, correction=correction)
 	IPM <- Tmatrix + Fmatrix
@@ -1918,7 +2012,8 @@ sensParams <- function(growObj,survObj,fecObj,
 	for (param.test in 1:length(survObj@fit$coeff)){
 		survObj@fit$coefficients[param.test] <- survObj@fit$coefficients[param.test]*(1+delta);
 		Tmatrix <- createIPMTmatrix(nBigMatrix = nBigMatrix, minSize=minSize,maxSize=maxSize,
-				growObj=growObj,survObj=survObj, integrateType=integrateType, correction=correction)
+				growObj=growObj,survObj=survObj, discreteTrans=discreteTrans,
+				integrateType=integrateType, correction=correction)
 		Fmatrix <- createIPMFmatrix(nBigMatrix = nBigMatrix, minSize=minSize,maxSize=maxSize,
 				fecObj=fecObj,integrateType=integrateType, correction=correction)
 		IPM <- Tmatrix+Fmatrix
@@ -1934,19 +2029,20 @@ sensParams <- function(growObj,survObj,fecObj,
 	
 	
 	#change the constant fecundity objects
-	chs <- which(!is.na(fecObj@fecConstants), arr.ind=TRUE)
+	chs <- which(!is.na(fecObj@fecConstants), arr.ind=TRUE)[,2]
 	if (length(chs)>0) { 
 		count <- count + param.test;
 		for (param.test in 1:length(chs)) {
 			fecObj@fecConstants[chs[param.test]] <- fecObj@fecConstants[chs[param.test]]*(1+delta);
 			Tmatrix <- createIPMTmatrix(nBigMatrix = nBigMatrix, minSize=minSize,maxSize=maxSize,
-					growObj=growObj,survObj=survObj, integrateType=integrateType, correction=correction)
+					growObj=growObj,survObj=survObj, discreteTrans=discreteTrans,
+					integrateType=integrateType, correction=correction)
 			Fmatrix <- createIPMFmatrix(nBigMatrix = nBigMatrix, minSize=minSize,maxSize=maxSize,
 					fecObj=fecObj,integrateType=integrateType, correction=correction)
 			IPM <- Tmatrix+Fmatrix
 			lambda2 <- Re(eigen(IPM)$value[1]);
-			fecObj@fecConstants[chs[param.test]] <- fecObj@fecConstants[chs[param.test]]/(1+delta);
-			slam[param.test+count]<-(lambda2-lambda1)/(fecObj@fecConstants[chs[param.test]]*delta);
+			fecObj@fecConstants[1,chs[param.test]] <- fecObj@fecConstants[1,chs[param.test]]/(1+delta);
+			slam[param.test+count]<-(lambda2-lambda1)/as.numeric(fecObj@fecConstants[1,chs[param.test]]*delta);
 			elam[param.test+count]<-(lambda2-lambda1)/(log(1+delta));
 			
 		}
@@ -1959,21 +2055,21 @@ sensParams <- function(growObj,survObj,fecObj,
 	count <-  param.test + count;
 	for (i in 1:length(fecObj@fitFec)){
 		#print(i)
-	for (param.test in 1:length(fecObj@fitFec[[i]]$coefficients)){
-		#print(param.test)
-		fecObj@fitFec[[i]]$coefficients[param.test] <- fecObj@fitFec[[i]]$coefficients[param.test]*(1+delta);
-		#print(fecObj@fitFec[[i]]$coefficients)
-		#print(cpar)
-		Fmatrix1 <- createIPMFmatrix(nBigMatrix = nBigMatrix, minSize=minSize,maxSize=maxSize,
-				fecObj=fecObj,integrateType=integrateType, correction=correction)
-				
-		IPM <- Tmatrix+Fmatrix1
-		lambda2 <- Re(eigen(IPM)$value[1]);#print(lambda2)
-		fecObj@fitFec[[i]]$coefficients[param.test] <- fecObj@fitFec[[i]]$coefficients[param.test]/(1+delta);
-		slam[param.test+count]<-(lambda2-lambda1)/(fecObj@fitFec[[i]]$coefficients[param.test]*delta);
-		elam[param.test+count]<-(lambda2-lambda1)/(log(1+delta));		
-	}
-	count <- count + param.test;
+		for (param.test in 1:length(fecObj@fitFec[[i]]$coefficients)){
+			#print(param.test)
+			fecObj@fitFec[[i]]$coefficients[param.test] <- fecObj@fitFec[[i]]$coefficients[param.test]*(1+delta);
+			#print(fecObj@fitFec[[i]]$coefficients)
+			#print(cpar)
+			Fmatrix1 <- createIPMFmatrix(nBigMatrix = nBigMatrix, minSize=minSize,maxSize=maxSize,
+					fecObj=fecObj,integrateType=integrateType, correction=correction)
+			
+			IPM <- Tmatrix+Fmatrix1
+			lambda2 <- Re(eigen(IPM)$value[1]);#print(lambda2)
+			fecObj@fitFec[[i]]$coefficients[param.test] <- fecObj@fitFec[[i]]$coefficients[param.test]/(1+delta);
+			slam[param.test+count]<-(lambda2-lambda1)/(fecObj@fitFec[[i]]$coefficients[param.test]*delta);
+			elam[param.test+count]<-(lambda2-lambda1)/(log(1+delta));		
+		}
+		count <- count + param.test;
 	}	
 	
 	#print("after fec ")
@@ -2049,8 +2145,15 @@ stochGrowthRateManyCov <- function(covariate,nRunIn,tMax,
 	
 	
 	for (t in 1:tMax) {
-		if (dd) tmp.fecObj@fecConstants <- c(fecObj@fecConstants, 
-					min(nMicrosites[min(t,length(nMicrosites))]/nt[1],1))
+		#if (dd) tmp.fecObj@fecConstants <- c(fecObj@fecConstants, 
+		#			min(nMicrosites[min(t,length(nMicrosites))]/nt[1],1))
+		
+		#changed for new fecConstants defintion
+		
+		
+		if (dd) tmp.fecObj@fecConstants$continuous <-  
+					min(nMicrosites[min(t,length(nMicrosites))]/nt[1],1)
+		#print(tmp.fecObj@fecConstants)
 		
 		
 		tpS <- createIPMTmatrix(nBigMatrix = nBigMatrix, minSize = minSize,
@@ -2105,8 +2208,11 @@ trackPopStructManyCov<-function(covariate,nRunIn,tMax,
 	
 	
 	for (t in 1:tMax) {
-		if (dd) tmp.fecObj@fecConstants <- c(fecObj@fecConstants, 
-					min(nMicrosites[min(t,length(nMicrosites))]/nt[1],1))
+		#if (dd) tmp.fecObj@fecConstants <- c(fecObj@fecConstants, 
+		#			min(nMicrosites[min(t,length(nMicrosites))]/nt[1],1))
+		if (dd) tmp.fecObj@fecConstants$prob.est <-  
+					min(nMicrosites[min(t,length(nMicrosites))]/nt[1],1)
+
 		
 		#print(tmp.fecObj@fecConstants)
 		
