@@ -378,14 +378,15 @@ makeSurvObjManyCov <- function(dataf,
 # 3. fecundity models  #######################################################################################################
 
 makeFecObj <- function(dataf,
-		fecConstants=as.numeric(NA),
+		fecConstants=data.frame(NA),
 		explanatoryVariables="size",
 		Family="gaussian",
 		Transform="none",
 		fecNames=NA,
 		meanOffspringSize=NA,
-		varOffspringSize=NA,
+		sdOffspringSize=NA,
 		offspringSplitter=data.frame(continuous=1),
+		offspringTypeRates=data.frame(NA),
 		fecByDiscrete=data.frame(NA)){
 	
 	#order stage names from discrete to continuous
@@ -461,12 +462,25 @@ makeFecObj <- function(dataf,
 	
 	if (is.na(meanOffspringSize)) {
 		offspringdata<-subset(dataf,is.na(dataf$stage)&dataf$stageNext=="continuous")
-		meanOffspringSize <- mean(offspringdata$sizeNext)
-		varOffspringSize <- var(offspringdata$sizeNext) }
+		meanOffspringSize <- mean(offspringdata$sizeNext, na.rm=TRUE)
+		sdOffspringSize <- sd(offspringdata$sizeNext, na.rm=TRUE) }
+	
+	if (sum(dim(offspringTypeRates)==c(1,1))<2) {
+		if ((sum(offspringTypeRates==0,na.rm=T)+sum(offspringTypeRates==1,na.rm=T))<(ncol(offspringTypeRates)*nrow(offspringTypeRates))) stop("Error - in offspringTypeRates data.frame only 0's and 1's are allowed: a 1 indicates that a fecundity rate applies to that offspring type. ")
+		if (sum(names(offspringTypeRates)==names(offspringSplitter))<length(offspringSplitter)) stop("Error - the offspring names in offspringTypeRates should match those in offspringSplitter - and in the same order, with continuous last")
+		if (sum(rownames(offspringTypeRates)==c(fecNames,names(fecConstants)))<(length(fecNames)+length(fecConstants))) stop ("Error - the row names in offspringTypeRates should consist of (in order) the names of the fec columns in the dataset and then the names of the fecConstants.")
+	} else {
+		offspringTypeRates <- as.data.frame(matrix(1,ncol=length(offspringSplitter),nrow=length(fecNames)+length(fecConstants)),row.names=c(fecNames,names(fecConstants)))
+		names(offspringTypeRates) <- names(offspringSplitter)
+	}
+	
+	
+	f1@fecNames <- fecNames
 	f1@fecConstants <- fecConstants
 	f1@meanOffspringSize <- meanOffspringSize
-	f1@varOffspringSize <- varOffspringSize
+	f1@sdOffspringSize <- sdOffspringSize
 	f1@offspringSplitter <- offspringSplitter 
+	f1@offspringTypeRates <- offspringTypeRates 
 	f1@fecByDiscrete <- fecByDiscrete
 	f1@Transform <- Transform
 	return(f1)
@@ -477,13 +491,14 @@ makeFecObj <- function(dataf,
 ## NO different from the above yet, except in what it produces
 
 makeFecObjManyCov <- function(dataf,
-		fecConstants=as.numeric(NA),
+		fecConstants=data.frame(NA),
 		explanatoryVariables="size",
 		Family="gaussian",
 		Transform="none",
 		meanOffspringSize=NA,
-		varOffspringSize=NA,
+		sdOffspringSize=NA,
 		offspringSplitter=data.frame(continuous=1),
+		offspringTypeRates=data.frame(NA),
 		fecByDiscrete=data.frame(NA)){
 	
 	##warnings
@@ -545,11 +560,230 @@ makeFecObjManyCov <- function(dataf,
 	if (is.na(meanOffspringSize)) {
 		offspringdata<-subset(dataf,is.na(dataf$stage)&dataf$stageNext=="continuous")
 		meanOffspringSize <- mean(offspringdata$sizeNext)
-		varOffspringSize <- var(offspringdata$sizeNext) }
+		sdOffspringSize <- sd(offspringdata$sizeNext) }
+	
+	if (sum(dim(offspringTypeRates)==c(1,1))<2) {
+		if ((sum(offspringTypeRates==0,na.rm=T)+sum(offspringTypeRates==1,na.rm=T))<(ncol(offspringTypeRates)*nrow(offspringTypeRates))) stop("Error - in offspringTypeRates data.frame only 0's and 1's are allowed: a 1 indicates that a fecundity rate applies to that offspring type. ")
+		if (sum(names(offspringTypeRates)==names(offspringSplitter))<length(offspringSplitter)) stop("Error - the offspring names in offspringTypeRates should match those in offspringSplitter - and in the same order, with continuous last")
+		if (sum(rownames(offspringTypeRates)==c(fecNames,names(fecConstants)))<(length(fecNames)+length(fecConstants))) stop ("Error - the row names in offspringTypeRates should consist of (in order) the names of the fec columns in the dataset and then the names of the fecConstants.")
+	} else {
+		offspringTypeRates <- as.data.frame(matrix(1,ncol=length(offspringSplitter),nrow=length(fecNames)+length(fecConstants)),row.names=c(fecNames,names(fecConstants)))
+		names(offspringTypeRates) <- names(offspringSplitter)
+	}
+	
 	f1@fecConstants <- fecConstants
 	f1@meanOffspringSize <- meanOffspringSize
-	f1@varOffspringSize <- varOffspringSize
+	f1@sdOffspringSize <- sdOffspringSize
 	f1@offspringSplitter <- offspringSplitter 
+	f1@offspringTypeRates <- offspringTypeRates
+	f1@fecByDiscrete <- fecByDiscrete
+	f1@Transform <- Transform
+	return(f1)
+}
+
+
+
+
+# 3a. clonality models  #######################################################################################################
+
+makeClonalObj <- function(dataf,
+		fecConstants=data.frame(NA),
+		explanatoryVariables="size",
+		Family="gaussian",
+		Transform="none",
+		fecNames=NA,
+		meanOffspringSize=NA,
+		sdOffspringSize=NA,
+		offspringSplitter=data.frame(continuous=1),
+		offspringTypeRates=data.frame(NA),
+		fecByDiscrete=data.frame(NA)){
+	
+	#order stage names from discrete to continuous
+	stages <- names(tapply(c(levels(dataf$stage),levels(dataf$stageNext)),c(levels(dataf$stage),levels(dataf$stageNext)),length))
+	stages <- stages[stages!="dead"] 
+	stages <- c(stages[stages!="continuous"],"continuous") 
+	if ((sum(names(offspringSplitter)%in%stages)/length(offspringSplitter))<1) {
+		stop("Error - the variable names in your offspringSplitter data.frame are not all part of the levels of stage or stageNext in your data file. Please fix this by adjusting your offspringSplitter entry to include the correct variable names, e.g. offspringSplitter=data.frame(continuous=.7,seedAge1=.3)")
+	}
+	dummy<-rep(0,length(stages));names(dummy)<-stages;dummy<-as.data.frame(t(as.matrix(dummy)))
+	for (i in names(offspringSplitter)) dummy[i]<-offspringSplitter[i]
+	offspringSplitter <- dummy
+	
+	##warnings
+	if (length(dataf$stage)==0) {
+		print("Warning - no column named stage - assuming all continuous")
+		dataf$stageNext <- dataf$stage <- rep("continuous", length(dataf[,1]))
+		dataf$stage[is.na(dataf$size)] <- NA
+		dataf$stageNext[is.na(dataf$sizeNext)] <- "dead"
+	}
+	
+	if (ncol(offspringSplitter)>1 & (ncol(offspringSplitter)-1)!=ncol(fecByDiscrete)) {
+		print("Warning - offspring splitter indicates more than just continuous stages. No fecundity by the discrete stages supplied in fecByDiscrete; assumed that is 0")
+		fecByDiscrete <- offspringSplitter[,1:(ncol(offspringSplitter)-1)]
+		fecByDiscrete[] <- 0
+	}
+	
+	if (sum(offspringSplitter)!=1) {
+		print("Warning - offspring splitter does not sum to 1. It is now rescaled to sum to 1.")
+		offspringSplitter <- offspringSplitter / sum(offspringSplitter) 
+	}
+	
+	if ("covariate"%in%strsplit(as.character(explanatoryVariables),"[+-\\*]")[[1]]&length(dataf$covariate)>0) { 
+		dataf$covariate <- as.factor(dataf$covariate)
+		levels(dataf$covariate) <- 1:length(unique(dataf$covariate))
+		
+	}
+	if ("covariateNext"%in%strsplit(as.character(explanatoryVariables),"[+-\\*]")[[1]]&length(dataf$covariateNext)>0) { 
+		dataf$covariateNext <- as.factor(dataf$covariateNext)
+		levels(dataf$covariateNext) <- 1:length(unique(dataf$covariateNext))
+	}
+	#print(table(dataf$covariate))
+	
+	f1 <- new("fecObj")
+	dataf$size2 <- dataf$size^2
+	if (length(grep("logsize",as.character(explanatoryVariables)))>0) dataf$logsize <- log(dataf$size)
+	
+	if (is.na(fecNames)) fecNames <- names(dataf)[grep("fec",names(dataf))]
+	if (length(fecNames)>length(explanatoryVariables)) {
+		misE <- (length(explanatoryVariables)+1):length(fecNames)
+		print(c("number in explanatoryVariables not the same as the number of fecundity columns in the data file, using default of `size' for missing ones which are:",fecNames[misE],". (which might be exactly what you want)"))
+		explanatoryVariables <- c(explanatoryVariables,rep("size",length(fecNames)-length(explanatoryVariables)))
+	}
+	if (length(fecNames)>length(Family)) {
+		misE <- (length(Family)+1):length(fecNames)
+		print(c("number of families not the same as the number of fecundity columns in the data file, using default of `gaussian' for missing ones which are:",fecNames[misE],". (which might be exactly what you want)"))
+		Family <- c(Family,rep("gaussian",length(fecNames)-length(Family)))
+	}
+	if (length(fecNames)>length(Transform)) {
+		misE <- (length(Transform)+1):length(fecNames)
+		print(c("number of transforms not the same as the number of fecundity columns in the data file, using default of `none' for missing ones which are:",fecNames[misE],". (which might be exactly what you want)"))
+		Transform <- c(Transform,rep("none",length(fecNames)-length(Transform)))
+	}
+	
+	for (i in 1:length(fecNames)) {
+		if (Transform[i]=="log") dataf[,fecNames[i]] <- log(dataf[,fecNames[i]])
+		if (Transform[i]=="sqrt") dataf[,fecNames[i]] <- sqrt(dataf[,fecNames[i]])
+		if (Transform[i]=="-1") dataf[,fecNames[i]] <- dataf[,fecNames[i]]-1
+		dataf[!is.finite(dataf[,fecNames[i]]),fecNames[i]] <- NA
+		f1@fitFec[[i]] <- glm(paste(fecNames[i],'~',explanatoryVariables[i],sep=''),family=Family[i],data=dataf)
+	}
+	
+	if (is.na(meanOffspringSize)) {
+		offspringdata<-subset(dataf,is.na(dataf$stage)&dataf$stageNext=="continuous")
+		meanOffspringSize <- mean(offspringdata$sizeNext, na.rm=TRUE)
+		sdOffspringSize <- sd(offspringdata$sizeNext, na.rm=TRUE) }
+	
+	if (sum(dim(offspringTypeRates)==c(1,1))<2) {
+		if ((sum(offspringTypeRates==0,na.rm=T)+sum(offspringTypeRates==1,na.rm=T))<(ncol(offspringTypeRates)*nrow(offspringTypeRates))) stop("Error - in offspringTypeRates data.frame only 0's and 1's are allowed: a 1 indicates that a fecundity rate applies to that offspring type. ")
+		if (sum(names(offspringTypeRates)==names(offspringSplitter))<length(offspringSplitter)) stop("Error - the offspring names in offspringTypeRates should match those in offspringSplitter - and in the same order, with continuous last")
+		if (sum(rownames(offspringTypeRates)==c(fecNames,names(fecConstants)))<(length(fecNames)+length(fecConstants))) stop ("Error - the row names in offspringTypeRates should consist of (in order) the names of the fec columns in the dataset and then the names of the fecConstants.")
+	} else {
+		offspringTypeRates <- as.data.frame(matrix(1,ncol=length(offspringSplitter),nrow=length(fecNames)+length(fecConstants)),row.names=c(fecNames,names(fecConstants)))
+		names(offspringTypeRates) <- names(offspringSplitter)
+	}
+	
+	
+	f1@fecNames <- fecNames
+	f1@fecConstants <- fecConstants
+	f1@meanOffspringSize <- meanOffspringSize
+	f1@sdOffspringSize <- sdOffspringSize
+	f1@offspringSplitter <- offspringSplitter 
+	f1@offspringTypeRates <- offspringTypeRates 
+	f1@fecByDiscrete <- fecByDiscrete
+	f1@Transform <- Transform
+	return(f1)
+}
+
+
+
+## NO different from the above yet, except in what it produces
+
+makeClonalObjManyCov <- function(dataf,
+		fecConstants=data.frame(NA),
+		explanatoryVariables="size",
+		Family="gaussian",
+		Transform="none",
+		meanOffspringSize=NA,
+		sdOffspringSize=NA,
+		offspringSplitter=data.frame(continuous=1),
+		offspringTypeRates=data.frame(NA),
+		fecByDiscrete=data.frame(NA)){
+	
+	##warnings
+	if (length(dataf$stage)==0) {
+		print("Warning - no column named stage - assuming all continuous")
+		dataf$stageNext <- dataf$stage <- rep("continuous", length(dataf[,1]))
+		dataf$stage[is.na(dataf$size)] <- NA
+		dataf$stageNext[is.na(dataf$sizeNext)] <- "dead"
+	}
+	
+	if(ncol(offspringSplitter)>1 & (ncol(offspringSplitter)-1)!=ncol(fecByDiscrete)) {
+		print("Warning - offspring splitter indicates more than just continuous stages. No fecundity by the discrete stages supplied in fecByDiscrete; assumed that is 0")
+		fecByDiscrete <- matrix(0,col(offspringSplitter)-1,col(offspringSplitter)-1)
+	}
+	
+	if(sum(offspringSplitter)!=1) {
+		print("Warning - offspring splitter does not sum to 1. It is now rescaled to sum to 1.")
+		
+	}
+	
+	# the covariate covariatenext transform removed here, because presumably if you are doing ManyCov you 
+	# have something else in mind!
+	
+	f1 <- new("fecObjMultiCov")
+	dataf$size2 <- dataf$size^2
+	if (length(grep("logsize",explanatoryVariables))>0) dataf$logsize <- log(dataf$size)
+	
+	fecNames <- names(dataf)[grep("fec",names(dataf))]
+	if (length(fecNames)>length(explanatoryVariables)) {
+		misE <- length(explanatoryVariables):length(fecNames)
+		print(c("number in explanatoryVariables not the same as the number of fecundity columns in the data file, using default of `size' for missing ones which are:",fecNames[misE]))
+		explanatoryVariables <- c(explanatoryVariables,rep("size",length(fecNames)-length(explanatoryVariables)))
+	}
+	if (length(fecNames)>length(Family)) {
+		misE <- length(Family):length(fecNames)
+		print(c("number of families not the same as the number of fecundity columns in the data file, using default of `gaussian' for missing ones which are:",fecNames[misE]))
+		Family <- c(Family,rep("gaussian",length(fecNames)-length(Family)))
+	}
+	if (length(fecNames)>length(Transform)) {
+		misE <- length(Transform):length(fecNames)
+		print(c("number of transforms not the same as the number of fecundity columns in the data file, using default of `none' for missing ones which are:",fecNames[misE]))
+		Transform <- c(Transform,rep("none",length(fecNames)-length(Transform)))
+	}
+	
+	for (i in 1:length(fecNames)) {
+		
+		if (Transform[i]=="log") dataf[,fecNames[i]] <- log(dataf[,fecNames[i]])
+		if (Transform[i]=="sqrt") dataf[,fecNames[i]] <- sqrt(dataf[,fecNames[i]])
+		if (Transform[i]=="-1") dataf[,fecNames[i]] <- dataf[,fecNames[i]]-1
+		dataf[!is.finite(dataf[,fecNames[i]]),fecNames[i]] <- NA
+		
+		#print(range(dataf[,fecNames[i]]))
+		#print(range(dataf[,"size"], na.rm=TRUE))
+		#print(range(dataf[,"covariate"]))
+		
+		f1@fitFec[[i]] <- glm(paste(fecNames[i],'~',explanatoryVariables[i],sep=''),family=Family[i],data=dataf)
+	}
+	
+	if (is.na(meanOffspringSize)) {
+		offspringdata<-subset(dataf,is.na(dataf$stage)&dataf$stageNext=="continuous")
+		meanOffspringSize <- mean(offspringdata$sizeNext)
+		sdOffspringSize <- sd(offspringdata$sizeNext) }
+	
+	if (sum(dim(offspringTypeRates)==c(1,1))<2) {
+		if ((sum(offspringTypeRates==0,na.rm=T)+sum(offspringTypeRates==1,na.rm=T))<(ncol(offspringTypeRates)*nrow(offspringTypeRates))) stop("Error - in offspringTypeRates data.frame only 0's and 1's are allowed: a 1 indicates that a fecundity rate applies to that offspring type. ")
+		if (sum(names(offspringTypeRates)==names(offspringSplitter))<length(offspringSplitter)) stop("Error - the offspring names in offspringTypeRates should match those in offspringSplitter - and in the same order, with continuous last")
+		if (sum(rownames(offspringTypeRates)==c(fecNames,names(fecConstants)))<(length(fecNames)+length(fecConstants))) stop ("Error - the row names in offspringTypeRates should consist of (in order) the names of the fec columns in the dataset and then the names of the fecConstants.")
+	} else {
+		offspringTypeRates <- as.data.frame(matrix(1,ncol=length(offspringSplitter),nrow=length(fecNames)+length(fecConstants)),row.names=c(fecNames,names(fecConstants)))
+		names(offspringTypeRates) <- names(offspringSplitter)
+	}
+	
+	f1@fecConstants <- fecConstants
+	f1@meanOffspringSize <- meanOffspringSize
+	f1@sdOffspringSize <- sdOffspringSize
+	f1@offspringSplitter <- offspringSplitter 
+	f1@offspringTypeRates <- offspringTypeRates
 	f1@fecByDiscrete <- fecByDiscrete
 	f1@Transform <- Transform
 	return(f1)
@@ -578,44 +812,44 @@ makeDiscreteTrans <- function(dataf) {
 	#define the number of classes
 	nclasses <- length(stages)
 	#define matrices to hold the transition between all classes
-	discrete.trans <- matrix(0,nrow=nclasses,ncol=nclasses, dimnames=list(stages,stages))
+	discreteTrans <- matrix(0,nrow=nclasses,ncol=nclasses, dimnames=list(stages,stages))
 	#define matrix to hold sd and mean of re-entry into continous + matrix of  survival for all discrete stages
-	sd.to.cont <- mean.to.cont <- discrete.surv <- matrix(NA,nrow=1,ncol=nclasses-1,dimnames=list(1,stages[1:length(stages)-1]))
+	sdToCont <- meanToCont <- discreteSurv <- matrix(NA,nrow=1,ncol=nclasses-1,dimnames=list(1,stages[1:length(stages)-1]))
 	# define matrix to hold transitions from the continuous to the discrete
-	distrib.to.discrete <- matrix(NA,ncol=1,nrow=nclasses-1,dimnames=list(stages[1:length(stages)-1],"continuous"))
+	distribToDiscrete <- matrix(NA,ncol=1,nrow=nclasses-1,dimnames=list(stages[1:length(stages)-1],"continuous"))
 	
 	#loop over discrete stages and fill 
 	for (j in stages[1:(length(stages)-1)]) {
-		for (i in stages) discrete.trans[i,j] <- sum(dataf[dataf$stage==j & dataf$stageNext==i,]$number,na.rm=TRUE)
-		discrete.surv[,j] <- sum(discrete.trans[,j],na.rm=T) / sum(dataf[dataf$stage == j,]$number, na.rm = TRUE)
-		discrete.trans[,j] <- discrete.trans[,j] / sum(discrete.trans[,j], na.rm = TRUE)
-		mean.to.cont[,j] <- mean(dataf[dataf$stage == j & dataf$stageNext == i,]$sizeNext, na.rm = TRUE)
-		sd.to.cont[,j] <- sd(dataf[dataf$stage == j & dataf$stageNext == i, ]$sizeNext,na.rm = TRUE)
+		for (i in stages) discreteTrans[i,j] <- sum(dataf[dataf$stage==j & dataf$stageNext==i,]$number,na.rm=TRUE)
+		discreteSurv[,j] <- sum(discreteTrans[,j],na.rm=T) / sum(dataf[dataf$stage == j,]$number, na.rm = TRUE)
+		discreteTrans[,j] <- discreteTrans[,j] / sum(discreteTrans[,j], na.rm = TRUE)
+		meanToCont[,j] <- mean(dataf[dataf$stage == j & dataf$stageNext == i,]$sizeNext, na.rm = TRUE)
+		sdToCont[,j] <- sd(dataf[dataf$stage == j & dataf$stageNext == i, ]$sizeNext,na.rm = TRUE)
 	}
 	
 	for (i in stages[1:(length(stages) - 1)])
-		distrib.to.discrete[i,] <- sum(dataf[dataf$stage == "continuous" & dataf$stageNext == i,]$number, na.rm = TRUE)
-	distrib.to.discrete <- distrib.to.discrete/sum(distrib.to.discrete,na.rm=TRUE)
+		distribToDiscrete[i,] <- sum(dataf[dataf$stage == "continuous" & dataf$stageNext == i,]$number, na.rm = TRUE)
+	distribToDiscrete <- distribToDiscrete/sum(distribToDiscrete,na.rm=TRUE)
 	
 	
 	subdata <- subset(dataf, dataf$stage == "continuous" & dataf$surv == 1)
 	subdata$cont.to.discrete <- 1
 	subdata$cont.to.discrete[subdata$stageNext == "continuous"] <- 0
 	subdata$size2 <- subdata$size ^ 2
-	surv.to.discrete <- glm(cont.to.discrete ~ size + size2, family = binomial, data = subdata)
+	survToDiscrete <- glm(cont.to.discrete ~ size + size2, family = binomial, data = subdata)
 			
-	rownames(discrete.trans) <- stages	
-	colnames(discrete.trans) <- stages	
+	rownames(discreteTrans) <- stages	
+	colnames(discreteTrans) <- stages	
 	
 	#define new object
 	disTrans <- new("discreteTrans")
 	disTrans@nclasses <- nclasses
-	disTrans@discrete.trans <- discrete.trans
-	disTrans@discrete.surv <- discrete.surv
-	disTrans@mean.to.cont <- mean.to.cont
-	disTrans@sd.to.cont <- sd.to.cont
-	disTrans@distrib.to.discrete <- distrib.to.discrete
-	disTrans@surv.to.discrete <- surv.to.discrete
+	disTrans@discreteTrans <- discreteTrans
+	disTrans@discreteSurv <- discreteSurv
+	disTrans@meanToCont <- meanToCont
+	disTrans@sdToCont <- sdToCont
+	disTrans@distribToDiscrete <- distribToDiscrete
+	disTrans@survToDiscrete <- survToDiscrete
 	
 	return(disTrans)
 	
@@ -819,13 +1053,14 @@ makePostSurvivalObjs <- function(dataf,
 #
 # Returns - list including list of growth objects, + list of survival objects
 makePostFecObjs <- function(dataf,
-		fecConstants=as.numeric(NA),
+		fecConstants=data.frame(NA),
 		explanatoryVariables="size",
 		Family="gaussian",
 		Transform="none",
 		meanOffspringSize=NA,
-		varOffspringSize=NA,
+		sdOffspringSize=NA,
 		offspringSplitter=data.frame(continuous=1),
+		offspringTypeRates=data.frame(NA),
 		fecByDiscrete=data.frame(NA),burnin=3000,nitt=50000) {
 	
 	require(MCMCglmm)
@@ -843,7 +1078,7 @@ makePostFecObjs <- function(dataf,
 		fecByDiscrete <- matrix(0,col(offspringSplitter)-1,col(offspringSplitter)-1)
 	}
 	
-    #setup for discrete covariates if data suggests may be implemented by the
+	#setup for discrete covariates if data suggests may be implemented by the
 	#presence of "covariate" and "covariateNext"
 	if ("covariate"%in%strsplit(as.character(explanatoryVariables),"[+-\\*]")[[1]]&length(dataf$covariate)>0) { 
 		dataf$covariate <- as.factor(dataf$covariate)
@@ -854,7 +1089,7 @@ makePostFecObjs <- function(dataf,
 		dataf$covariateNext <- as.factor(dataf$covariateNext)
 		levels(dataf$covariateNext) <- 1:length(unique(dataf$covariateNext))
 	}
-			
+	
 	dataf$size2 <- dataf$size^2
 	if (length(grep("logsize",explanatoryVariables))>0) dataf$logsize <- log(dataf$size)
 	fecNames <- names(dataf)[grep("fec",names(dataf))]
@@ -873,8 +1108,8 @@ makePostFecObjs <- function(dataf,
 	
 	if (is.na(meanOffspringSize)) {
 		offspringdata<-subset(dataf,is.na(dataf$stage)&dataf$stageNext=="continuous")
-		meanOffspringSize <- mean(offspringdata$sizeNext)
-		varOffspringSize <- var(offspringdata$sizeNext)
+		meanOffspringSize <- mean(offspringdata$sizeNext,na.rm=TRUE)
+		sdOffspringSize <- sd(offspringdata$sizeNext,na.rm=TRUE)
 	}
 	
 	#get rid of NAs
@@ -901,6 +1136,17 @@ makePostFecObjs <- function(dataf,
 		
 	}
 	
+	if (sum(dim(offspringTypeRates)==c(1,1))<2) {
+		if ((sum(offspringTypeRates==0,na.rm=T)+sum(offspringTypeRates==1,na.rm=T))<(ncol(offspringTypeRates)*nrow(offspringTypeRates))) stop("Error - in offspringTypeRates data.frame only 0's and 1's are allowed: a 1 indicates that a fecundity rate applies to that offspring type. ")
+		if (sum(names(offspringTypeRates)==names(offspringSplitter))<length(offspringSplitter)) stop("Error - the offspring names in offspringTypeRates should match those in offspringSplitter - and in the same order, with continuous last")
+		if (sum(rownames(offspringTypeRates)==c(fecNames,names(fecConstants)))<(length(fecNames)+length(fecConstants))) stop ("Error - the row names in offspringTypeRates should consist of (in order) the names of the fec columns in the dataset and then the names of the fecConstants.")
+	} else {
+		offspringTypeRates <- as.data.frame(matrix(1,ncol=length(offspringSplitter),nrow=length(fecNames)+length(fecConstants)),row.names=c(fecNames,names(fecConstants)))
+		names(offspringTypeRates) <- names(offspringSplitter)
+	}
+	
+	
+	
 	#print(length(fit[[1]]$Sol[,1]))	
 	
 	#create list of growth models reflecing posterior
@@ -913,13 +1159,14 @@ makePostFecObjs <- function(dataf,
 			##TODO check over-ride
 			dummy.fit[[i]]$residuals <- rnorm(length(dummy.fit[[i]]$residuals),0,sqrt(fit[[i]]$VCV[k,1]))
 			fv[[k]]@fitFec[[i]] <- dummy.fit[[i]]
-					
+			
 		}
 		
 		fv[[k]]@fecConstants <- fecConstants
 		fv[[k]]@meanOffspringSize <- meanOffspringSize
-		fv[[k]]@varOffspringSize <- varOffspringSize
+		fv[[k]]@sdOffspringSize <- sdOffspringSize
 		fv[[k]]@offspringSplitter <- offspringSplitter
+		fv[[k]]@offspringTypeRates <- offspringTypeRates
 		fv[[k]]@fecByDiscrete <- fecByDiscrete
 		fv[[k]]@Transform <- Transform 
 	}
