@@ -305,7 +305,7 @@ picGrow <- function(dataf,growObj) {
 	sizes <- dataf$size[!is.na(dataf$size)]; sizes <- sizes[order(sizes)]
 	
 	for (k in 1:length(ud)) { 
-		newd <- data.frame(size=sizes,size2=sizes^2,
+		newd <- data.frame(size=sizes,size2=sizes^2,size3=sizes^3,
 				covariate=as.factor(rep(ud[k],length(sizes))))
 					
 		if(length(grep("logsize",names(growObj@fit$coefficients)))==1)
@@ -472,7 +472,7 @@ makeListIPMs <- function(dataf, nBigMatrix=10, minSize=-2,maxSize=10,
 		integrateType="midpoint", correction="none",
 		explSurv="size+size2+covariate",explGrow="size+size2+covariate", 
 		regType="constantVar",responseType="sizeNext",explFec="size",Family="gaussian", 
-		Transform="none",fecConstants=1) {
+		Transform="none",fecConstants=data.frame(NA)) {
 	
 	#convert to 1:n for indexing later
 	dataf$covariate <- as.factor(dataf$covariate)
